-- Some changes to the mixerWagon specialization to allow for different products to be input/output
-- slight changes to animalHusbandry with regards to pigs to make use of the pig food mixer
-- put together by alfalfa6945
-- version 0.1.0.0

changedFunctions = {}
changedFunctions.modDir = g_currentModDirectory

function changedFunctions:loadMap(name)
	changedFunctions:loadXMLvalue()
end

function changedFunctions:loadXMLvalue()
	local path = getUserProfileAppPath()
	local file, xmlFile = path.."modsSettings/changedPigFeeding.xml"
	if fileExists(file) then
		xmlFile = loadXMLFile("changedPigFeedingXML", file)
	else
		xmlFile = createXMLFile("changedPigFeedingXML", file, "changedValues")
		local title = {"fillTypeGroup_pigFoodMix", "fillTypeGroup_base", "fillTypeGroup_earth"}
		local productionWeight = {1.0, 0.75, 0.25}
		local fillTypes = {"PIGFOOD", "MAIZE", "POTATO SUGARBEET"}
		for i = 1, #title do
			local groupKey = string.format(".foodGroup(%d)", i - 1)
			setXMLString(xmlFile, "changedValues"..groupKey.."#title", title[i])
			setXMLFloat(xmlFile, "changedValues"..groupKey.."#productionWeight", productionWeight[i])
			setXMLString(xmlFile, "changedValues"..groupKey.."#fillTypes", fillTypes[i])
		end
		saveXMLFile(xmlFile)
	end
	if xmlFile then
		g_animalFoodManager.foodGroups.PIG.consumptionType = AnimalFoodManager.FOOD_CONSUME_TYPE_SERIAL
		g_animalFoodManager.foodGroups.PIG.content = {}
		g_animalFoodManager.foodMixtures[FillType.PIGFOOD] = nil
		g_animalFoodManager.animalFoodMixtures.PIG = nil
		local i = 0
		while true do
			local groupKey = string.format("changedValues.foodGroup(%d)", i)
			if not hasXMLProperty(xmlFile, groupKey) then
				break
			end
			local foodGroup = {}
			local title = Utils.getNoNil(getXMLString(xmlFile, groupKey.."#title"), "")
			foodGroup.title = g_i18n:hasText(title) and g_i18n:getText(title) or title
			foodGroup.productionWeight = Utils.getNoNil(getXMLFloat(xmlFile, groupKey.."#productionWeight"), 0.0)
			foodGroup.eatWeight = Utils.getNoNil(getXMLFloat(xmlFile, groupKey.."#eatWeight"), 1.0)
			foodGroup.fillTypes = {}
			local fillTypesStr = Utils.getNoNil(getXMLString(xmlFile, groupKey.."#fillTypes"), "")
			local warning = string.format("Warning: FillType '%s' is undefined, ignoring it!", tostring(fillTypeName))
			foodGroup.fillTypes = g_fillTypeManager:getFillTypesByNames(fillTypesStr, warning)
			table.insert(g_animalFoodManager.foodGroups.PIG.content, foodGroup)
			i = i + 1
		end
		delete(xmlFile)
	end
end

addModEventListener(changedFunctions)

MixerWagon.newOnLoad = function(self, savegame)
	local spec = self.spec_mixerWagon
	local mixType = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.mixerWagon#mixedFillType"), "forage")
	spec.mixedFillType = g_fillTypeManager:getFillTypeIndexByName(mixType)
end
MixerWagon.onLoad = Utils.prependedFunction(MixerWagon.onLoad, MixerWagon.newOnLoad)

function MixerWagon:addFillUnitFillLevel(superFunc, farmId, fillUnitIndex, fillLevelDelta, fillTypeIndex, toolType, fillPositionData)
    local spec = self.spec_mixerWagon
    if fillUnitIndex ~= spec.fillUnitIndex then
        return superFunc(self, farmId, fillUnitIndex, fillLevelDelta, fillTypeIndex, toolType, fillPositionData)
    end
    local oldFillLevel = self:getFillUnitFillLevel(fillUnitIndex)
    local mixerWagonFillType = spec.fillTypeToMixerWagonFillType[fillTypeIndex]
	if fillTypeIndex == spec.mixedFillType and fillLevelDelta > 0 then
        for _, entry in pairs(spec.mixerWagonFillTypes) do
            local delta = fillLevelDelta * entry.ratio
            self:addFillUnitFillLevel(farmId, fillUnitIndex, delta, next(entry.fillTypes), toolType, fillPositionData)
        end
        return fillLevelDelta
    end
    if mixerWagonFillType == nil then
        if fillLevelDelta < 0 and oldFillLevel > 0 then
            fillLevelDelta = math.max(fillLevelDelta, -oldFillLevel)
            local newFillLevel = 0
            for _, entry in pairs(spec.mixerWagonFillTypes) do
                local entryDelta = fillLevelDelta * (entry.fillLevel / oldFillLevel)
                entry.fillLevel = math.max(entry.fillLevel + entryDelta, 0)
                newFillLevel = newFillLevel + entry.fillLevel
            end
            self:raiseDirtyFlags(spec.dirtyFlag)
            local ret = superFunc(self, farmId, fillUnitIndex, fillLevelDelta, fillTypeIndex, toolType, fillPositionData)
            return ret
        end
        return 0
    end
    local capacity = self:getFillUnitCapacity(fillUnitIndex)
    local free = capacity - oldFillLevel
    if fillLevelDelta > 0 then
        mixerWagonFillType.fillLevel = mixerWagonFillType.fillLevel + math.min(free, fillLevelDelta)
        spec.activeTimer = spec.activeTimerMax
    else
        mixerWagonFillType.fillLevel = math.max(0, mixerWagonFillType.fillLevel + fillLevelDelta)
    end
    local newFillLevel = 0
    for _, mixerWagonFillType in pairs(spec.mixerWagonFillTypes) do
        newFillLevel = newFillLevel + mixerWagonFillType.fillLevel
    end
    newFillLevel = MathUtil.clamp(newFillLevel, 0, self:getFillUnitCapacity(fillUnitIndex))
    local newFillType = FillType.UNKNOWN
    local isSingleFilled = false
    local isForageOk = false
    for _, mixerWagonFillType in pairs(spec.mixerWagonFillTypes) do
        if newFillLevel == mixerWagonFillType.fillLevel then
            isSingleFilled = true
            newFillType = next(mixerWagonFillType.fillTypes)
            break
        end
    end
    if not isSingleFilled then
        isForageOk = true
        for _, mixerWagonFillType in pairs(spec.mixerWagonFillTypes) do
            if mixerWagonFillType.fillLevel < mixerWagonFillType.minPercentage * newFillLevel - 0.01 or mixerWagonFillType.fillLevel > mixerWagonFillType.maxPercentage * newFillLevel + 0.01 then
                isForageOk = false
                break
            end
        end
    end
    if isForageOk then
		newFillType = spec.mixedFillType
    elseif not isSingleFilled then
        newFillType = FillType.FORAGE_MIXING
    end
    self:raiseDirtyFlags(spec.dirtyFlag)
    self:setFillUnitFillType(fillUnitIndex, newFillType)
    return superFunc(self, farmId, fillUnitIndex, newFillLevel-oldFillLevel, newFillType, toolType, fillPositionData)
end